/** 
  \file elan.h
  \brief Declaration of the class to handle the XML file created in Elan software.
  \version 2.0
  \date July 2012
  \author Tomasz Kapuscinski (tomekkap@prz-rzeszow.pl) and Dawid Warchol (dawwar@o2.pl)
 */

#ifndef __ELAN_H
#define __ELAN_H

#include <string>
#include <vector>
#include "tier.h"
#include "elanQueryResult.h"

using namespace std;

class CElan
{
	public:
		static string ANNOTATION_DOCUMENT_ENTRY;
		static string HEADER_ENTRY;
		static string MEDIA_DESCRIPTOR_ENTRY;
		static string PROPERTY_ENTRY;
		static string TIME_SLOT_ENTRY;
		static string TIER_ENTRY;
		static string ANNOTATION_ENTRY;
		static string REF_ANNOTATION_ENTRY;
		static string ANNOTATION_VALUE_ENTRY;
		static string LINGUISTIC_TYPE_ENTRY;
		static string CONTROLLED_VOCABULARY_ENTRY;
		static string CV_ENTRY_ENTRY;
		static string LOCALE_ENTRY;
		static string CONSTRAINT_ENTRY;

		static string MOVIE_EXTENSION;
		static string ELAN_EXTENSION;

		CElan(int p_iFrameRate);
		CElan(string p_szFile, int p_iFrameRate);
		void Load(string p_szFile);
		void Save(string p_szFile);
		int GetFrameRate(){return m_iFrameRate;}
		void ClearData();
		vector<CElanQueryResult> QueryElan(string p_szTierName, int p_iFieldIndex, string p_szFieldValue);
		vector<CElanQueryResult> QueryElan(string p_szTierName, int p_iStartFrame, int p_iStopFrame);
		void Print();
		void AddTier(string p_szTierName);
		void AddTier(CTier p_oTier);
		CTier GetTier(string p_szTierName);
		CTier GetTier(int p_iTierIndex);
		CTier& GetTierRef(string p_szTierName);
		CTier& GetTierRef(int p_iTierIndex);
		vector<CTier> GetTiers(){return m_oTiers;}
		vector<CTier>& GetTiersRef(){return m_oTiers;}
		void InsertTier(string p_szTierName, CTier p_oTier, bool p_bInsertAfter=false);
		void InsertTier(int p_iTierIndex, CTier p_oTier, bool p_bInsertAfter=false);
		void ReplaceTier(string p_szTierName, CTier p_oTier);
		void ReplaceTier(int p_iTierIndex, CTier p_oTier);
		void DeleteTier(string p_szTierName);
		void DeleteTier(int p_iTierIndex);
		void DeleteAllTiers() {m_oTiers.clear();}
		CTier CompareTiers(CTier p_oFirstTier, CTier p_oSecondTier, string p_szComparisonTierName, int p_iFieldIndex, 
						   string p_szValueIfEqual, string p_szValueIfDifferent, bool p_bMergeIdenticalAnnotations);
		CTier CompareTiers(string p_szFirstTierName, string p_szSecondTierName, string p_szComparisonTierName, int p_iFieldIndex, 
						   string p_szValueIfEqual, string p_szValueIfDifferent, bool p_bMergeIdenticalAnnotations);
		int GetTiersNumber() {return (int)m_oTiers.size();}
		void AddAnnotation(string p_szTierName, CAnnotation p_oAnnotation);
		CAnnotation GetAnnotation(string p_szTierName, int p_iAnnotationIndex);
		vector<CAnnotation> GetAnnotations(string p_szTierName);
		CAnnotation& GetAnnotationRef(string p_szTierName, int p_iAnnotationIndex);
		vector<CAnnotation>& GetAnnotationsRef(string p_szTierName);
		CTier MergeIdenticalAnnotations(CTier p_oTier, int p_iFieldIndex);
		CTier MergeIdenticalAnnotations(string p_szTierName, int p_iFieldIndex);
		string GetDocumentDate() {return m_szDocumentDate;}
		void AddMovieFile(string p_szFile);
		void DeleteMovieFile(string p_szFile);
		void DeleteMovieFile(int p_iPosition);
		void DeleteAllMovieFiles();
		vector<string> GetMovieFiles() {return m_oRelativeMediaURLs;}
		vector<string> GetMoviePaths() {return m_oMediaURLs;}
		vector<string> GetLinguisticTypeIDs() {return m_oLinguisticTypeIDs;}
		static string MovieFile2ElanFile(string p_szMovieFile);
		static vector<CElanQueryResult> Concatenate(vector<CElanQueryResult> p_oVector1, vector<CElanQueryResult> p_oVector2);
		void TierToSubtitles(string p_szFile, string p_szTierName, string p_szFieldsSeparator=";");
		void TierToSubtitles(string p_szFile, string p_szTierName, int p_iFieldIndex);
		void Export(string p_szFile);

	private:
		bool IsEntry(string p_szTextLine, string p_szEntry);

		bool IsAnnotationDocumentEntry(string p_szTextLine);
		bool IsHeaderEntry(string p_szTextLine);
		bool IsMediaDescriptorEntry(string p_szTextLine);
		bool IsPropertyEntry(string p_szTextLine);
		bool IsTimeSlotEntry(string p_szTextLine);
		bool IsTierEntry(string p_szTextLine);
		bool IsAnnotationEntry(string p_szTextLine);
		bool IsRefAnnotationEntry(string p_szTextLine);
		bool IsAnnotationValueEntry(string p_szTextLine);
		bool IsLinguisticTypeEntry(string p_szTextLine);
		bool IsControlledVocabularyEntry(string p_szTextLine);
		bool IsCVEntryEntry(string p_szTextLine);
		bool IsLocaleEntry(string p_szTextLine);
		bool IsConstraintEntry(string p_szTextLine);

		string GetAttribute(string p_szTextLine, string p_szAttributeName);

		string GetDocumentAuthor(string p_szTextLine);
		string GetDocumentDate(string p_szTextLine);
		string GetDocumentFormat(string p_szTextLine);
		string GetDocumentVersion(string p_szTextLine);
		string GetDocumentXMLNS(string p_szTextLine);
		string GetDocumentNoNamespaceSchemaLocation(string p_szTextLine);
		string GetHeaderMediaFile(string p_szTextLine);
		string GetHeaderTimeUnits(string p_szTextLine);
		string GetMediaURL(string p_szTextLine);
		string GetMimeType(string p_szTextLine);
		string GetExtractedFrom(string p_szTextLine);
		string GetRelativeMediaURL(string p_szTextLine);
		string GetPropertyName(string p_szTextLine);
		string GetGraphicReferences(string p_szTextLine);
		string GetLinguisticTypeID(string p_szTextLine);
		string GetTimeAlignable(string p_szTextLine);
		string GetLinguisticConstraints(string p_szTextLine);
		string GetControlledVobabularyRef(string p_szTextLine);
		string GetCountryCode(string p_szTextLine);
		string GetLanguageCode(string p_szTextLine);
		string GetConstraintDescription(string p_szTextLine);
		string GetConstraintStereotype(string p_szTextLine);
		string GetCVID(string p_szTextLine);
		string GetCVDescription(string p_szTextLine);
		string GetCVEntryDescription(string p_szTextLine);

		int GetTimeSlot(string p_szTextLine);
		int GetAnnotationRef(string p_szTextLine);
		string GetTierName(string p_szTextLine);
		string GetDefaultLocale(string p_szTextLine);
		string GetLinguisticType(string p_szTextLine);
		string GetAnnotator(string p_szTextLine);
		string GetParticipant(string p_szTextLine);
		string GetParentRef(string p_szTextLine);
		int GetAnnotationTime(string p_szTextLint, bool p_bStart);
		string GetAnnotationTimeString(string p_szTextLint, bool p_bStart);
		int GetAnnotationStart(string p_szTextLine, int p_iFrameRate);
		int GetAnnotationStop(string p_szTextLine, int p_iFrameRate);
		string GetNodeValue(string p_szTextLine);
		string GetLocalTimeAndDate();
		string GetLocalTimeZone();
		string GetMonthNumber(string oMonthName);

		void SortIndexes(vector<int> &p_oAnnotationIndexes, vector<int> &p_oWhoseTierWhichIndex);
		void RemoveRepeatedTimeSlots(vector <int> &p_oTimeSlots);
		string BackslashesToSlashes(string p_szFile);
		string FileNameFromPath(string p_szFile);
		string EafToAvi(string p_szFile);

		string m_szAviFile;
		string m_szEafFile;

		const int m_iFrameRate;
		vector<int> m_oTimeSlots;
		vector<CTier> m_oTiers;
	public:
		string m_szDocumentAuthor;
		string m_szDocumentDate;
		string m_szDocumentFormat;
		string m_szDocumentVersion;
		string m_szDocumentXMLNS;
		string m_szDocumentNoNamespaceSchemaLocation;
		string m_szHeaderMediaFile;
		string m_szHeaderTimeUnits;
		vector<string> m_oPropertyNames;
		vector<string> m_oPropertyValues;

		vector<string> m_oGraphicReferences;
		vector<string> m_oLinguisticTypeIDs;
		vector<string> m_oTimeAlignables;
		vector<string> m_oLinguisticConstraints;
		vector<string> m_oControlledVocabularyRefs;
		vector<string> m_oCountryCodes;
		vector<string> m_oLanguageCodes;
		vector<string> m_oConstraintDescriptions;
		vector<string> m_oConstraintStereotypes;
		vector<string> m_oMediaURLs;
		vector<string> m_oMimeTypes;
		vector<string> m_oRelativeMediaURLs;
		vector<string> m_oExtractedFroms;

		struct SControlledVocabulary
		{
			string m_szID;
			string m_szDescription;
			vector<string> m_oEntryDescriptions;
			vector<string> m_oEntryValues;
		};

		vector<SControlledVocabulary> m_oControlledVocabularies;
};


#endif
